#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..settings import Settings
from logging.handlers import TimedRotatingFileHandler
from io import StringIO
import logging
import time
import os
import sys


class Logger:

    def __init__(self, logger_name="punittest",
                 logger_dir=Settings.LOG_DIR,
                 console_level=Settings.LOG_CONSOLE_LEVEL,
                 file_level=Settings.LOG_FILE_LEVEL,
                 report_level=Settings.LOG_REPORT_LEVEL):
        """
        :param logger_dir: 日志文件所在目录路径
        :param logger_name: logger名称
        :param console_level: 控制台输出级别
        :param file_level: 文件输出级别
        :param report_level: 报告输出级别
        """
        self.logger = logging.getLogger(logger_name)
        self.logger.log_capture = None
        self.logger_dir = logger_dir
        logging.root.setLevel(logging.NOTSET)
        self.formatter = logging.Formatter(
            # '[%(asctime)s][%(levelname)s][%(module)s][%(funcName)s][%(lineno)d]: %(message)s')
            '[%(asctime)s][%(levelname)s][%(threadName)s]: %(message)s')
        self.console_output_level = console_level
        self.file_output_level = file_level
        self.report_output_level = report_level
        self.backup_count = 20

    def get_logger(self, console_switch=Settings.LOG_CONSOLE_SWITCH,
                   file_switch=Settings.LOG_FILE_SWITCH,
                   report_switch=Settings.LOG_REPORT_SWITCH):
        """
        在logger中添加日志句柄并返回，如果logger已有句柄，则直接返回
        :param console_switch: 控制台日志开关
        :param file_switch: 文件日志开关
        :param report_switch: 报告日志开关
        :return:
        """
        if not self.logger.handlers:
            if console_switch:
                console_handler = logging.StreamHandler(sys.stdout)
                console_handler.setLevel(self.console_output_level)
                console_handler.setFormatter(self.formatter)
                self.logger.addHandler(console_handler)
            if file_switch:
                now_time = time.strftime("%Y-%m-%d_%H-%M-%S")
                log_file_name = '{0}.log'.format(now_time)
                if not os.path.exists(self.logger_dir):
                    os.makedirs(self.logger_dir)
                log_file_path = os.path.join(self.logger_dir, log_file_name)
                file_handler = TimedRotatingFileHandler(filename=log_file_path, when='D', interval=1,
                                                        backupCount=self.backup_count, delay=True, encoding='utf-8')
                file_handler.setLevel(self.file_output_level)
                file_handler.setFormatter(self.formatter)
                self.logger.addHandler(file_handler)
            if report_switch:
                # 将Log放入StringIO，最后由HTMLTestRunner读取
                self.logger.log_capture = StringIO()
                report_handler = logging.StreamHandler(self.logger.log_capture)
                report_handler.setLevel(self.report_output_level)
                report_handler.setFormatter(self.formatter)
                self.logger.addHandler(report_handler)
        return self.logger


logger = Logger().get_logger()



#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from .utils.logger import logger
import time
import io
import sys
import unittest


class TestResult(unittest.TestResult):
    def __init__(self, verbosity=1):
        super().__init__(verbosity)
        # 根据report的log开关取不同的值
        if logger.log_capture is None:
            self.outputBuffer = io.StringIO()
        else:
            self.outputBuffer = logger.log_capture
        self.raw_stdout = None
        self.raw_stderr = None
        self.success_count = 0
        self.failure_count = 0
        self.skip_count = 0
        self.error_count = 0
        self.verbosity = verbosity
        self.result = []
        self._case_start_time = 0
        self._case_run_time = 0

    def startTest(self, test):
        self._case_start_time = time.time()
        super().startTest(test)
        # 将StringIO重定向到sys.stdout
        self.raw_stdout = sys.stdout
        self.raw_stderr = sys.stderr
        sys.stdout = self.outputBuffer
        sys.stderr = self.outputBuffer

    def complete_output(self):
        self._case_run_time = time.time() - self._case_start_time
        if self.raw_stdout:
            sys.stdout = self.raw_stdout
            sys.stderr = self.raw_stderr
        # 从StringIO中读取log信息
        result = self.outputBuffer.getvalue()
        lines = result.splitlines()
        # 移除每条测试用例的开始和结束标记，只留下内容
        start_idx, end_idx = 0, 0
        for i, line in enumerate(lines):
            if "* Start *" in line:
                start_idx = i
            elif "* End *" in line:
                end_idx = i
        if end_idx > start_idx:
            result = '\n'.join(lines[start_idx + 1: end_idx])
            result += '\n'
        # 移动StringIO的偏移指针，读取新的IO内容
        self.outputBuffer.seek(0)
        self.outputBuffer.truncate()
        return result

    def stopTest(self, test):
        self.complete_output()

    def addSuccess(self, test):
        self.success_count += 1
        super().addSuccess(test)
        output = self.complete_output()
        self.result.append((0, test, output, '', self._case_run_time))

    def addError(self, test, err):
        self.error_count += 1
        super().addError(test, err)
        _, _exc_str = self.errors[-1]
        output = self.complete_output()
        self.result.append((2, test, output, _exc_str, self._case_run_time))

    def addSkip(self, test, reason):
        self.skip_count += 1
        super().addSkip(test, reason)
        self.result.append((3, test, "", "", 0.0))

    def addFailure(self, test, err):
        self.failure_count += 1
        super().addFailure(test, err)
        _, _exc_str = self.failures[-1]
        output = self.complete_output()
        self.result.append((1, test, output, _exc_str, self._case_run_time))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .settings import Settings as SETTINGS

__all__ = ['SETTINGS']

if SETTINGS.__SPECIFIED__ is True:
    
    from .punittest import PUnittest, logger
    from .testrunner import TestRunner
    from .testresult import TestResult
    
    __all__.extend(['PUnittest', 'TestRunner', 'TestResult', 'logger'])


def RELOAD():
    SETTINGS.__SPECIFIED__ = True
    from importlib import reload
    import punittest
    reload(punittest)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from punittest import PUnittest, logger


class TestPurchase(PUnittest):
    """测试游戏内购买功能"""

    def setUp(self):
        super(TestPurchase, self).setUp()
        logger.info("登录角色")
        logger.info("修改角色货币")

    def tearDown(self):
        logger.info("角色登出")
        super(TestPurchase, self).tearDown()

    @PUnittest.tag("Smoke", "Regression")
    def test_purchase_weapon(self):
        """测试购买武器成功"""
        logger.info("调用购买接口，获取购买结果")
        result = "某款武器"
        self.assertEqual(result, "某款武器")

    @PUnittest.tag("Smoke", "Regression")
    def test_purchase_armor(self):
        """测试购买防具成功"""
        logger.info("调用购买接口，获取购买结果")
        result = "某款防具"
        self.assertTrue(result, "某款饰品")

    @PUnittest.tag()
    def test_purchase_accessories(self):
        """测试购买饰品成功"""
        logger.info("将角色货币归0")
        logger.info("调用购买接口，获取购买结果")
        result = "角色货币不足"
        self.assertTrue(result, "某款饰品")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from punittest import PUnittest, logger


class TestLogin(PUnittest):
    """测试登录功能"""

    @PUnittest.tag("Smoke", "Regression")
    @PUnittest.data([{'user': 'username', 'pass': 'password'}], [True])
    def test_login_successfully(self, params, asserts):
        """测试登录成功"""
        _user, _passwd = params['user'], params['pass']
        _result = asserts
        logger.info("调用登录接口，用户名/密码: {}/{}， 获取登录结果".format(_user, _passwd))
        result = True
        self.assertTrue(_result, result)

    @PUnittest.tag("Regression")
    @PUnittest.data(
        [{'user': 'username', 'pass': '123'},
         {'user': '123', 'pass': 'password'},
         {'user': '', 'pass': ''}],
        [False, False, False])
    def test_login_failed(self, params, asserts):
        """测试登录失败"""
        _user, _passwd = params['user'], params['pass']
        _result = asserts
        logger.info("调用登录接口，用户名/密码: {}/{}， 获取登录结果".format(_user, _passwd))
        result = False
        self.assertTrue(_result, result)

    @PUnittest.skip("这里跳过不测")
    @PUnittest.tag("Regression")
    @PUnittest.data([{'user': 'username', 'pass': 'password'}], [True])
    def test_relogin_successfully(self, params, asserts):
        """测试重新登录成功"""
        _user, _passwd = params['user'], params['pass']
        _result = asserts
        logger.info("调用登录接口，用户名/密码: {}/{}， 获取登录结果".format(_user, _passwd))
        result = True
        self.assertTrue(_result, result)
        logger.info("调用登录接口，用户名/密码: {}/{}， 获取登录结果".format(_user, _passwd))
        result = True
        self.assertTrue(_result, result)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .testcase import Meta
from .testcase import skip, skip_if, tag, data
from .utils.logger import logger
import unittest


class PUnittest(unittest.TestCase, metaclass=Meta):

    Exc_Stack = []
    skip = skip
    skip_if = skip_if
    tag = tag
    data = data

    def setUp(self):
        logger.info(' Start '.center(100, '*'))

    def tearDown(self):
        logger.info(' End '.center(100, '*'))
        logger.info('')

    def _clear_exc_stack(self):
        self.Exc_Stack.clear()

    def _raise_exc(self):
        if self.Exc_Stack:
            exc_text = ', AssertionError: '.join(str(x) for x in self.Exc_Stack)
            self._clear_exc_stack()
            raise AssertionError(exc_text)

    def assertEqual(self, first, second, msg=None):
        super(PUnittest, self).assertEqual(first, second, msg=msg)

    def assertAlmostEqual(self, first, second, places=None, msg=None, delta=None):
        super(PUnittest, self).assertAlmostEqual(first, second, places=places, msg=msg, delta=delta)

    def assertDictEqual(self, d1, d2, msg=None):
        super(PUnittest, self).assertDictEqual(d1=d1, d2=d2, msg=msg)

    def assertFalse(self, expr, msg=None):
        super(PUnittest, self).assertFalse(expr, msg=msg)

    def assertTrue(self, expr, msg=None):
        super(PUnittest, self).assertTrue(expr, msg=msg)

    def assertIn(self, member, container, msg=None):
        super(PUnittest, self).assertIn(member, container, msg=msg)

    def assertIs(self, expr1, expr2, msg=None):
        super(PUnittest, self).assertIs(expr1, expr2, msg=msg)

    def assertIsInstance(self, obj, cls, msg=None):
        super(PUnittest, self).assertIsInstance(obj, cls, msg=msg)

    def assertIsNone(self, obj, msg=None):
        super(PUnittest, self).assertIsNone(obj, msg=msg)

    def assertNotIn(self, member, container, msg=None):
        super(PUnittest, self).assertNotIn(member, container, msg=msg)

    def assertNotEqual(self, first, second, msg=None):
        super(PUnittest, self).assertNotEqual(first, second, msg=msg)

    def assertGreater(self, a, b, msg=None):
        super(PUnittest, self).assertGreater(a, b, msg=msg)

    def assertLess(self, a, b, msg=None):
        super(PUnittest, self).assertLess(a, b, msg=msg)


if __name__ == '__main__':
    pass



#! /usr/bin/env python
# -*- coding: UTF-8 -*-

import os

root_path = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
static_path = os.path.join(root_path, "static")
bootstrap_css = os.path.join(static_path, "bootstrap", "3.0.3", "css", "bootstrap.min.css")
jquery = os.path.join(static_path, "jquery", "2.0.0", "jquery.min.js")
bootstrap_js = os.path.join(static_path, "bootstrap", "3.0.3", "js", "bootstrap.min.js")

html_head = r"""
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title>{report_name}</title>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>""" \
    + """<link href="{}" rel="stylesheet">""".format(bootstrap_css) \
    + """<script src="{}"></script>""".format(jquery) \
    + """<script src="{}"></script>""".format(bootstrap_js) \
    + r"""
    <style type="text/css" media="screen">
        pre {{
            white-space: pre-wrap;
            white-space: -moz-pre-wrap;
            white-space: -pre-wrap;
            white-space: -o-pre-wrap;
            word-wrap: break-word;
            text-align: left;
        }}
    
        body {{
            font-family: Microsoft YaHei, Tahoma, arial, helvetica, sans-serif;
            padding: 20px;
            font-size: 80%;
        }}

        table {{
            font-size: 100%;
        }}

        /* -- heading ---------------------------------------------------------------------- */
        .heading {{
            margin-top: 0ex;
            margin-bottom: 1ex;
        }}

        .heading .description {{
            margin-top: 4ex;
            margin-bottom: 6ex;
        }}

        /* -- report_template ------------------------------------------------------------------------ */
        #total_row {{
            font-weight: bold;
        }}

        .passCase {{
            color: #5cb85c;
        }}

        .failCase {{
            color: #f0ad4e;
            font-weight: bold;
        }}

        .errorCase {{        
            color: #d9534f;
            font-weight: bold;
        }}

        .hiddenRow {{
            display: none;
        }}

        .testcase {{
            margin-left: 2em;
        }}
    </style>

</head>
<script language="javascript" type="text/javascript">
    output_list = Array();


    function showCase(level) {{
        trs = document.getElementsByTagName("tr");
        for (var i = 0; i < trs.length; i++) {{
            tr = trs[i];
            id = tr.id;
            if (id.substr(0, 2) == 'pt') {{
                if (level == 1 || level == 4) {{
                    tr.className = '';
                }}
                else {{
                    tr.className = 'hiddenRow';                    
                }}
            }}
            if (id.substr(0, 2) == 'ft') {{
                if (level == 2 || level == 4) {{
                    tr.className = '';
                }}
                else {{
                    tr.className = 'hiddenRow';
                }}
            }}
            
            if (id.substr(0, 2) == 'et') {{
                if (level == 3 || level == 4) {{
                    tr.className = '';
                }}
                else {{
                    tr.className = 'hiddenRow';
                }}
            }}
        }}

        detail_class = document.getElementsByClassName('detail');
        //console.log(detail_class.length)
        if (level == 3) {{
            for (var i = 0; i < detail_class.length; i++) {{
                detail_class[i].innerHTML = "收起"
            }}
        }}
        else {{
            for (var i = 0; i < detail_class.length; i++) {{
                detail_class[i].innerHTML = "详细"
            }}
        }}
    }}

    function showClassDetail(cid, count) {{
        var id_list = Array(count);
        var toHide = 1;
        for (var i = 0; i < count; i++) {{
            tid0 = 't' + cid.substr(1) + '_' + (i + 1);
            console.log(tid0);
            tid = 'f' + tid0;
            tr = document.getElementById(tid);
            if (!tr) {{
                tid = 'p' + tid0;
                tr = document.getElementById(tid);
            }}
            if (!tr) {{
                tid = 'e' + tid0;
                tr = document.getElementById(tid);
            }}
            id_list[i] = tid;
            if (tr.className) {{
                toHide = 0;
            }}
        }}
        for (var i = 0; i < count; i++) {{
            tid = id_list[i];
            if (toHide) {{
                document.getElementById(tid).className = 'hiddenRow';
                document.getElementById(cid).innerText = "详细"
            }}
            else {{
                document.getElementById(tid).className = '';
                document.getElementById(cid).innerText = "收起"
            }}
        }}
    }}

    function html_escape(s) {{
        s = s.replace(/&/g, '&amp;');
        s = s.replace(/</g, '&lt;');
        s = s.replace(/>/g, '&gt;');
        return s;
    }}
</script>
"""

html_body = """

<body>

<div class='heading'>
    <h1 style="font-family: Microsoft YaHei">{report_name}</h1>
    <p class='attribute'><strong>开始时间 : </strong> {begin_time}</p>
    <p class='attribute'><strong>合计耗时 : </strong> {total_time}</p>
    <p class='attribute'><strong>测试结果 : </strong> 共 {test_all}，通过 {test_pass}，失败 {test_fail}，错误 {test_error}，跳过 {test_skip}，通过率= {test_pass_per}</p>

    <p class='description'></p>
</div>


<p id='show_detail_line'>
    <a class="btn btn-primary" href='javascript:showCase(0)'>概要{{ {test_pass_per} }}</a>
    <a class="btn btn-success" href='javascript:showCase(1)'>通过{{ {test_pass} }}</a>
    <a class="btn btn-warning" href='javascript:showCase(2)'>失败{{ {test_fail} }}</a>
    <a class="btn btn-danger" href='javascript:showCase(3)'>错误{{ {test_error} }}</a>
    <a class="btn btn-info" href='javascript:showCase(4)'>所有{{ {test_all} }}</a>
</p>
<table id='result_table' class="table table-condensed table-bordered table-hover">
    <colgroup>
        <col align='left'/>
        <col align='right'/>
        <col align='right'/>
        <col align='right'/>
        <col align='right'/>
        <col align='right'/>
        <col align='right'/>
    </colgroup>
    <tr id='header_row' class="text-center success" style="font-weight: bold;font-size: 14px;">
        <td>用例集/测试用例</td>
        <td>总计</td>
        <td>通过</td>
        <td>失败</td>
        <td>错误</td>
        <td>跳过</td>
        <td>详细</td>
    </tr>

 
    {case_info}

    <tr id='total_row' class="text-center active">
        <td>总计</td>
        <td>{test_all}</td>
        <td>{test_pass}</td>
        <td>{test_fail}</td>
        <td>{test_error}</td>
        <td>{test_skip}</td>
        <td>通过率：{test_pass_per}</td>
    </tr>
</table>

<div id='ending'>&nbsp;</div>
<div style=" position:fixed;right:50px; bottom:30px; width:20px; height:20px;cursor:pointer">
    <a href="#"><span class="glyphicon glyphicon-eject" style="font-size:30px;" aria-hidden="true">
    </span></a></div>


</body>
</html>
"""

html_case_group = """
<tr class="failClass warning">
        <td>{group_name}</td>
        <td class="text-center">{test_all}</td>
        <td class="text-center">{test_pass}</td>
        <td class="text-center">{test_fail}</td>
        <td class="text-center">{test_error}</td>
        <td class="text-center">{test_skip}</td>
        <td class="text-center"><a href="javascript:showClassDetail('c{group_index}',{test_all})" class="detail" id="c{group_index}">详细</a></td>
    </tr>
"""

html_case_skip = """
<tr id="pt{case_id}" class="">
        <td class="passCase">
            <div class="testcase">{method_name}</div>
        </td>
        <td colspan="6" align="center">
        <span class="label label-info success">跳过</span>
        </td>
    </tr>
"""

html_case_pass_no_log = """
<tr id="pt{case_id}" class="">
        <td class="passCase">
            <div class="testcase">{method_name}</div>
        </td>
        <td colspan="6" align="center">
        <span class="label label-success success">通过</span>
        </td>
    </tr>
"""

html_case_pass_with_log = """
<tr id="pt{case_id}" class="">
        <td class="passCase">
            <div class="testcase">{method_name}</div>
        </td>
        <td colspan="6" align="center">
       
        <button id='btn_pt{case_id}' type="button"  class="btn btn-success btn-xs collapsed" data-toggle="collapse" data-target='#div_pt{case_id}'>通过</button>
        <div id='div_pt{case_id}' class="collapse">
                <pre>{case_log}</pre>
            </div>
        </td>
    </tr>
"""

html_case_fail = """
<tr id="ft{case_id}" class="">
        <td class="failCase">
            <div class="testcase">{method_name}</div>
        </td>
        <td colspan="6" align="center">
            <!--默认收起错误信息
            <button id='btn_ft1_2' type="button"  class="btn btn-danger btn-xs collapsed" data-toggle="collapse" data-target='#div_ft1_2'>失败</button>
            <div id='div_ft1_2' class="collapse">  -->

            <!-- 默认展开错误信息 -->
            <button id="btn_ft{case_id}" type="button" class="btn btn-warning btn-xs" data-toggle="collapse" data-target="#div_ft{case_id}">失败</button>
            <div id="div_ft{case_id}" class="collapse in">
                <pre>{case_log}</pre>
            </div>
        </td>
    </tr>
"""

html_case_error = """
<tr id="et{case_id}" class="">
    <td class="errorCase"><div class="testcase">{method_name}</div></td>
    <td colspan="6" align="center">
    <!--默认收起错误信息
    <button id='btn_et1_5' type="button"  class="btn btn-danger btn-xs collapsed" data-toggle="collapse" data-target='#div_et1_5'>错误</button>
    <div id='div_et1_5' class="collapse">  -->

    <!-- 默认展开错误信息 -->
    <button id="btn_et{case_id}" type="button" class="btn btn-danger btn-xs" data-toggle="collapse" data-target="#div_et{case_id}">错误</button>
    <div id="div_et{case_id}" class="collapse in">
        <pre>{case_log}</pre>
    </div>
    </td>
</tr>
"""


def build_report(file_path, data):
    html = html_head.format(report_name=data['reportName'])

    case_info = ""
    from collections import defaultdict
    filter_case = defaultdict(list)
    for i in data['testResult']:
        filter_case[i["className"]].append(i)

    for group_index, group in enumerate(filter_case.keys(), 1):
        pass_num = len([i for i in filter_case[group] if i['status'] == '成功'])
        fail_num = len([i for i in filter_case[group] if i['status'] == '失败'])
        error_num = len([i for i in filter_case[group] if i['status'] == '错误'])
        skip_num = len([i for i in filter_case[group] if i['status'] == '跳过'])
        case_info += html_case_group.format(group_name=group,
                                            group_index=group_index,
                                            test_all=pass_num + fail_num + error_num + skip_num,
                                            test_pass=pass_num,
                                            test_fail=fail_num,
                                            test_error=error_num,
                                            test_skip=skip_num
                                            )
        cases = filter_case[group]
        for case_index, case in enumerate(cases, 1):
            if case['status'] == '成功':
                if case['log']:
                    case_info += html_case_pass_with_log.format(case_id="{}_{}".format(group_index, case_index),
                                                                method_name="{}（{}）".format(case['methodName'],
                                                                                            case['description']),
                                                                case_log=case['log'])
                else:
                    case_info += html_case_pass_no_log.format(case_id="{}_{}".format(group_index, case_index),
                                                              method_name="{}（{}）".format(case['methodName'],
                                                                                          case['description']),
                                                              )
            elif case['status'] == '失败':
                case_info += html_case_fail.format(case_id="{}_{}".format(group_index, case_index),
                                                   method_name="{}（{}）".format(case['methodName'],
                                                                               case['description']),
                                                   case_log=case['log'])
            elif case['status'] == '错误':
                case_info += html_case_error.format(case_id="{}_{}".format(group_index, case_index),
                                                    method_name="{}（{}）".format(case['methodName'],
                                                                                case['description']),
                                                    case_log=case['log'])
            elif case['status'] == '跳过':
                case_info += html_case_skip.format(case_id="{}_{}".format(group_index, case_index),
                                                   method_name="{}（{}）".format(case['methodName'],
                                                                               case['description']),
                                                   )

    html += html_body.format(begin_time=data['beginTime'],
                             total_time=data['totalTime'],
                             test_all=data['testAll'],
                             test_pass=data['testPass'],
                             test_fail=data['testFail'],
                             test_skip=data['testSkip'],
                             test_error=data['testError'],
                             test_pass_per="{:.2%}".format(data['testPass'] / data['testAll']),
                             report_name=data['reportName'],
                             case_info=case_info,
                             )

    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(html)
